/*
 * CFile1.c
 *
 * Created: 20.01.2014 12:21:35
 *  Author: Alex
 */ 

#include "LTC2486_functions.h"

uint32_t LTC2486_SPI_get_value (uint16_t config){
	
	uint8_t buf_in;
	uint8_t buf_out;	
	uint32_t adc_receive =0;
	
	SPI_activate_slave();
	while(!LTC2486_conversion_complete); 				// wait until EOC Flag shows value ready

	buf_out = (config>>8);												
	SPI_write_byte(buf_out);							// send first (config) Byte
				
	while(!(SPSR & (1<<SPIF)));							// Wait until receiption complete;
	
	buf_in = SPI_read_byte();							// First byte received: Save first adc byte
	adc_receive |= ((uint32_t)buf_in <<16);				// Write into [1. Byte | HERE | 3. Byte | 4th Byte]
	buf_out = config;								
	SPI_write_byte(buf_out);							// send second (config) Byte

	while(!(SPSR & (1<<SPIF)));							// Wait until receiption complete;
	
	buf_in = SPI_read_byte();							// Second byte received: Save second adc byte
	adc_receive |= ((uint32_t)buf_in <<8);				// Write into [1. Byte | 2. Byte | HERE| 4th Byte]
	SPI_write_byte(0xFF);								// send dummy Byte for last reception

	while(!(SPSR & (1<<SPIF)));							// Wait until receiption complete;	
			
	buf_in = SPI_read_byte();							// Last byte received: Save last adc byte
	adc_receive |= buf_in;								// Write into [1. Byte | 2. Byte | 3. Byte | HERE]
	
	return adc_receive;
}

uint16_t LTC2486_create_config (uint8_t channel, bool doublespeed, bool temp, uint8_t gain) {
	uint16_t config;
	
	/* Set Start Bits, Enable bits, Single Ended mode */
	config = ((uint16_t)1<<15)|((uint16_t)1<<EN)|((uint16_t)1<<EN2)|((uint16_t)1<<SGL);
	
	/* Set Channel */
	switch (channel) {
		case 0: {
			config &= ~(((uint16_t)1<<ODD)|((uint16_t)1<<A0)|((uint16_t)1<<A1)|((uint16_t)1<<A2));
			break;
		}
		case 1: {
			config &= ~(((uint16_t)1<<A0)|((uint16_t)1<<A1)|((uint16_t)1<<A2));
			config |= ((uint16_t)1<<ODD);
			break;
		}
		case 2: {
			config &= ~(((uint16_t)1<<ODD)|((uint16_t)1<<A1)|((uint16_t)1<<A2));
			config |= ((uint16_t)1<<A0);
			break;
		}
		case 3: {
			config &= ~(((uint16_t)1<<A1)|((uint16_t)1<<A2));
			config |= ((uint16_t)1<<ODD)|((uint16_t)1<<A0);
			break;			
		}
		default: {	//channel 0
			config &= ~(((uint16_t)1<<ODD)|((uint16_t)1<<A0)|((uint16_t)1<<A1)|((uint16_t)1<<A2));
			break;			
		}		
	}
	
	/* Set Double Speed Mode */
	if (doublespeed == true) {
		config |= ((uint16_t)1<<SPD);
	}
	else {
		config &= ~((uint16_t)1<<SPD);
	}
	
	/* Set Temperature Sensor Measurement */
	if (temp == true) {
		config |= ((uint16_t)1<<IM);
	}
	else {
		config &= ~((uint16_t)1<<IM);
	}		
	
	switch (gain) {			// NOTE: In Double Speed mode, Gains are divided by two (except G=1)
		case 1: {
			config &= 0xFFF8;	// Set last three Bits (GS2:GS0) to zero
			break;
		}
		case 4: {
			config &= 0xFFF8;	// Set last three Bits (GS2:GS0) to zero
			config |= ((uint16_t)1<<GS0);
			break;
		}
		case 8: {
			config &= 0xFFF8;	// Set last three Bits (GS2:GS0) to zero
			config |= ((uint16_t)1<<GS1);
			break;
		}
		case 16: {
			config &= 0xFFF8;	// Set last three Bits (GS2:GS0) to zero
			config |= ((uint16_t)1<<GS0)|((uint16_t)1<<GS1);
			break;
		}
		case 32: {
			config &= 0xFFF8;	// Set last three Bits (GS2:GS0) to zero
			config |= ((uint16_t)1<<GS2);
			break;
		}
		case 64: {
			config &= 0xFFF8;	// Set last three Bits (GS2:GS0) to zero
			config |= ((uint16_t)1<<GS2)|((uint16_t)1<<GS0);
			break;
		}		
		case 128: {
			config &= 0xFFF8;	// Set last three Bits (GS2:GS0) to zero
			config |= ((uint16_t)1<<GS2)|((uint16_t)1<<GS1);
			break;
		}	
		case 255: {
			config &= 0xFFF8;	// Set last three Bits (GS2:GS0) to zero
			config |= ((uint16_t)1<<GS2)|((uint16_t)1<<GS1)|((uint16_t)1<<GS0);
			break;
		}
		default: {
			config &= 0xFFF8;	// Default: G=1;
		}																								
	}
	return config;
}

uint16_t LTC2486_extract_value (uint32_t data) {	// Simple 5 Bit-shifting, reads 16Bit ADC value from received ADC-Data-Block
	uint16_t adc_val=0;
	adc_val = (data>>5);
	return adc_val;	
}

uint16_t LTC2486_extract_temp (uint32_t data) {	// Simple 4 Bit-shifting, reads 16Bit temp value from received ADC-Data-Block
	uint16_t adc_val=0;
	adc_val = (data>>4);
	return adc_val;
}

bool LTC2486_conversion_complete (void)	{			// tests whether EOC-Flag (PB6) of ADC is set: If EOC=1: Still converting. If EOC=0:Data ready
	uint8_t port = PINB;
	if (!(port & (1<<PB6)))
	return true;
	else return false;
}

